@@ -30,9 +30,9 @@
 #' 
 #' }
 #'
-#' @param U48 a list containing three items (\code{x}, \code{sx} and
-#'     \code{option}) specifying the \eqn{{}^{234}}U/\eqn{{}^{238}}U
-#'     disequilibrium.
+#' @param U48 a list containing seven items (\code{x}, \code{sx},
+#'     \code{m}, \code{M}, \code{x0}, \code{sd} and \code{option})
+#'     specifying the \eqn{{}^{234}}U/\eqn{{}^{238}}U disequilibrium.
 #'
 #' If \code{option=0}, then \code{x} and \code{sx} are ignored and no
 #'     disequilibrium correction is applied.
@@ -45,9 +45,16 @@
 #'     \eqn{{}^{234}}U/\eqn{{}^{238}}U ratio and \code{sx} its
 #'     standard error.
 #'
-#' @param ThU a list containing three items (\code{x}, \code{sx} and
-#'     \code{option}) specifying the \eqn{{}^{230}}Th/\eqn{{}^{238}}U
-#'     disequilibrium.
+#' \code{m}, \code{M} specify the minimum and maximum search limits of
+#' the initial \eqn{{}^{234}}U/\eqn{{}^{238}}U activity ratio.
+#'
+#' \code{x0} and \code{sd} specify the mean and standard deviation of
+#' the prior distribution for the the initial
+#' \eqn{{}^{234}}U/\eqn{{}^{238}}U activity ratio.
+#'
+#' @param ThU a list containing seven items (\code{x}, \code{sx},
+#'     \code{m}, \code{M}, \code{x0}, \code{sd} and \code{option})
+#'     specifying the \eqn{{}^{230}}Th/\eqn{{}^{238}}U disequilibrium.
 #'
 #' If \code{option=0}, then \code{x} and \code{sx} are ignored and no
 #'     disequilibrium correction is applied.
@@ -65,9 +72,12 @@
 #'     volcanic glass) and \code{sx} its standard error. This only
 #'     applies for Th-bearing U-Pb data formats 7 and 8.
 #'
-#' @param RaU a list containing three items (\code{x}, \code{sx} and
-#'     \code{option}) specifying the \eqn{{}^{226}}Ra/\eqn{{}^{238}}U
-#'     disequilibrium.
+#' \code{m}, \code{M}, \code{x0} and \code{sd} are analogous to the
+#' eponymous settings for \code{ThU}.
+#'
+#' @param RaU a list containing seven items (\code{x}, \code{sx},
+#'     \code{m}, \code{M}, \code{x0}, \code{sd} and \code{option})
+#'     specifying the \eqn{{}^{226}}Ra/\eqn{{}^{238}}U disequilibrium.
 #'
 #' If \code{option=0}, then \code{x} and \code{sx} are ignored and no
 #'     disequilibrium correction is applied.
@@ -76,9 +86,12 @@
 #'     \eqn{{}^{226}}Ra/\eqn{{}^{238}}U ratio and \code{sx} its
 #'     standard error.
 #' 
-#' @param PaU a list containing three items (\code{x}, \code{sx} and
-#'     \code{option}) specifying the \eqn{{}^{231}}Pa/\eqn{{}^{235}}U
-#'     disequilibrium.
+#' \code{m}, \code{M}, \code{x0} and \code{sd} are analogous to the
+#' eponymous settings for \code{ThU}.
+#' 
+#' @param PaU a list containing seven items (\code{x}, \code{sx},
+#'     \code{m}, \code{M}, \code{x0}, \code{sd} and \code{option})
+#'     specifying the \eqn{{}^{231}}Pa/\eqn{{}^{235}}U disequilibrium.
 #'
 #' If \code{option=0}, then \code{x} and \code{sx} are ignored and no
 #'     disequilibrium correction is applied.
@@ -86,6 +99,13 @@
 #' If \code{option=1}, then \code{x} contains the initial
 #'     \eqn{{}^{231}}Pa/\eqn{{}^{235}}U ratio and \code{sx} its
 #'     standard error.
+#'
+#' \code{m}, \code{M}, \code{x0} and \code{sd} are analogous to the
+#' eponymous settings for \code{ThU}.
+#' 
+#' @param buffer small amount of padding to avoid singularities in the
+#'     prior distribution, which uses a logistic transformation:
+#' \eqn{y = \ln\left(\frac{x-m+buffer}{M+buffer-x}\right)}
 #' 
 #' @return a list with the following items:
 #'
@@ -101,12 +121,6 @@
 #' \item{Qinv}{the inverse of \code{Q}}
 #'
 #' \item{L}{a named vector of all the relevant decay constants}
-#'
-#' \item{n0}{the initial atomic abundances of all the parent and
-#' daughter isotopes (used by \code{\link{mclean}}). Note that the
-#' \eqn{{}^{238}}U and \eqn{{}^{235}}U decay chains have been
-#' disconnected, so the \eqn{{}^{238}}U/\eqn{{}^{235}}U ratio of
-#' \code{n0} is meaningless.}
 #' 
 #' }
 #'
@@ -119,16 +133,29 @@
 #' UPb <- read.data(fn,method='U-Pb',format=2,d=d)
 #' concordia(UPb,type=2,xlim=c(0,700),ylim=c(0.05,0.5))
 #' @export
-diseq <- function(U48=list(x=1,sx=0,option=0),
-                  ThU=list(x=1,sx=0,option=0),
-                  RaU=list(x=1,sx=0,option=0),
-                  PaU=list(x=1,sx=0,option=0)){
+diseq <- function(U48=list(x=1,sx=0,option=0,m=0,M=20,x0=1,sd=10),
+                  ThU=list(x=1,sx=0,option=0,m=0,M=20,x0=1,sd=10),
+                  RaU=list(x=1,sx=0,option=0,m=0,M=20,x0=1,sd=10),
+                  PaU=list(x=1,sx=0,option=0,m=0,M=20,x0=1,sd=10),
+                  buffer=1e-5){
+    patch <- function(aratio){
+        out <- aratio
+        if (is.null(aratio$x)) out$x <- 1
+        if (is.null(aratio$sx)) out$sx <- 0
+        if (is.null(aratio$option)) out$option <- 0
+        if (is.null(aratio$m)) out$m <- 0
+        if (is.null(aratio$M)) out$M <- 20
+        if (is.null(aratio$x0)) out$x0 <- 1
+        if (is.null(aratio$sd)) out$sd <- 10
+        out
+    }
     out <- list()
     class(out) <- 'diseq'
-    out$U48 <- U48
-    out$ThU <- ThU
-    out$RaU <- RaU
-    out$PaU <- PaU
+    out$U48 <- patch(U48)
+    out$ThU <- patch(ThU)
+    out$RaU <- patch(RaU)
+    out$PaU <- patch(PaU)
+    out$buffer <- buffer
     out$equilibrium <- check.equilibrium(d=out)
     l38 <- settings('lambda','U238')[1]
     l34 <- settings('lambda','U234')[1]*1000
@@ -136,6 +163,8 @@ diseq <- function(U48=list(x=1,sx=0,option=0),
     l26 <- settings('lambda','Ra226')[1]*1000
     l35 <- settings('lambda','U235')[1]
     l31 <- settings('lambda','Pa231')[1]*1000
+    out$L <- c(l38,l34,l30,l26,0,l35,l31,0)
+    names(out$L) <- c('U238','U234','Th230','Ra226','Pb206','U235','Pa231','Pb207')
     out$Q <- matrix(0,8,8)
     out$Q[1,1] <- -((l26-l38)*(l30-l38)*(l34-l38))/(l26*l30*l34)
     out$Q[2,1] <- -(l38*(l26-l38)*(l30-l38))/(l26*l30*l34)
@@ -168,11 +197,6 @@ diseq <- function(U48=list(x=1,sx=0,option=0),
     out$Qinv[7,6] <- l35 /(l31-l35)
     out$Qinv[7,7] <- -1
     out$Qinv[8,6:8] <- 1
-    out$L <- c(l38,l34,l30,l26,0,l35,l31,0)
-    out$n0 <- matrix(c(1/l38,1/l34,1/l30,1/l26,0,1/l35,1/l31,0),ncol=1)
-    nuclides <- c('U238','U234','Th230','Ra226','Pb206','U235','Pa231','Pb207')
-    names(out$L) <- nuclides
-    rownames(out$n0) <- nuclides
     out
 }
 
@@ -189,14 +213,6 @@ copy_diseq <- function(x,d=diseq()){
     out
 }
 
-geomean.diseq <- function(x,...){
-    out <- x
-    for (ratio in c('U48','ThU','RaU','PaU')){
-        out[[ratio]]$x <- geomean(x[[ratio]]$x,...)
-    }
-    out
-}
-
 # to infer initial 38,34,30,35 from measured 34/38 and 30/38
 mexp.8405 <- function(){
     l38 <- settings('lambda','U238')[1]
@@ -240,35 +256,38 @@ mexp.845 <- function(nratios=3){
 }
 
 reverse <- function(tt,mexp,nt){
-    out <- (mexp$Q %*% diag(exp(mexp$L*tt)) %*% mexp$Qinv %*% nt)
-    rownames(out) <- rownames(nt)
-    out
+    n0 <- (mexp$Q %*% diag(exp(mexp$L*tt)) %*% mexp$Qinv %*% nt)
+    rownames(n0) <- rownames(nt)
+    n0
 }
 forward <- function(tt,d=diseq(),derivative=0){
-    if (derivative==0){
-        out <- (d$Q %*% diag(exp(-d$L*tt)) %*% d$Qinv %*% d$n0)
-    } else if (derivative==1){
-        out <- (d$Q %*% diag(exp(-d$L)) %*%
-                diag(exp(-d$L*tt)) %*% d$Qinv %*% d$n0)
+    if (derivative==1){
+        nt <- (d$Q %*% diag(exp(-d$L)) %*%
+               diag(exp(-d$L*tt)) %*% d$Qinv %*% d$n0)
     } else if (derivative==2){
-        out <- (d$Q %*% diag(exp(-d$L)) %*% diag(exp(-d$L)) %*%
-                diag(exp(-d$L*tt)) %*% d$Qinv %*% d$n0)
+        nt <- (d$Q %*% diag(exp(-d$L)) %*% diag(exp(-d$L)) %*%
+               diag(exp(-d$L*tt)) %*% d$Qinv %*% d$n0)
+    } else {
+        nt <- (d$Q %*% diag(exp(-d$L*tt)) %*% d$Qinv %*% d$n0)
     }
-    rownames(out) <- names(d$L)
-    out
+    rownames(nt) <- names(d$L)
+    nt
 }
 
 check.equilibrium <- function(d=diseq()){
-    U48 <- (d$U48$option==0 | all(d$U48$x==1))
-    ThU <- (d$ThU$option==0 | all(d$ThU$x==1))
-    RaU <- (d$RaU$option==0 | all(d$RaU$x==1))
-    PaU <- (d$PaU$option==0 | all(d$PaU$x==1))
+    checkratio <- function(r){
+        r$option == 0 || all(r$x==1) & all(r$sx==0)
+    }
+    U48 <- checkratio(d$U48)
+    ThU <- checkratio(d$ThU)
+    RaU <- checkratio(d$RaU)
+    PaU <- checkratio(d$PaU)
     U48 & ThU & RaU & PaU
 }
+
 measured.disequilibrium <- function(d=diseq()){
-    equilibrium <- check.equilibrium(d)
     measured <- (d$U48$option==2 | d$ThU$option==2)
-    !equilibrium & measured
+    !d$equilibrium & measured
 }
 
 #' @title Predict disequilibrium concordia compositions
@@ -301,18 +320,20 @@ measured.disequilibrium <- function(d=diseq()){
 #' @param exterr propagate the uncertainties associated with decay
 #'     constants and the \eqn{{}^{238}}U/\eqn{{}^{235}}U-ratio.
 #'
-#' @return
-#' a list containing the predicted \eqn{{}^{206}}Pb/\eqn{{}^{238}}U,
-#' \eqn{{}^{207}}Pb/\eqn{{}^{235}}U and
-#' \eqn{{}^{207}}Pb/\eqn{{}^{206}}Pb ratios at time \code{tt}; the
-#' derivatives of the \eqn{{}^{206}}Pb/\eqn{{}^{238}}U,
-#' \eqn{{}^{207}}Pb/\eqn{{}^{235}}U and
-#' \eqn{{}^{207}}Pb/\eqn{{}^{206}}Pb ratios with respect to time; and
-#' the derivatives of the \eqn{{}^{206}}Pb/\eqn{{}^{238}}U,
-#' \eqn{{}^{207}}Pb/\eqn{{}^{235}}U and
-#' \eqn{{}^{207}}Pb/\eqn{{}^{206}}Pb ratios with respect to the
-#' intermediate decay constants and
-#' \eqn{{}^{238}}U/\eqn{{}^{235}}U-ratio.
+#' @return a list containing the initial and present-day atomic
+#'     abundances of the \eqn{{}^{238}}U-\eqn{{}^{206}}Pb and
+#'     \eqn{{}^{235}}U-\eqn{{}^{207}}Pb decay chains; the
+#'     \eqn{{}^{206}}Pb/\eqn{{}^{238}}U,
+#'     \eqn{{}^{207}}Pb/\eqn{{}^{235}}U and
+#'     \eqn{{}^{207}}Pb/\eqn{{}^{206}}Pb ratios at time \code{tt}; the
+#'     derivatives of the \eqn{{}^{206}}Pb/\eqn{{}^{238}}U,
+#'     \eqn{{}^{207}}Pb/\eqn{{}^{235}}U and
+#'     \eqn{{}^{207}}Pb/\eqn{{}^{206}}Pb ratios with respect to time;
+#'     and the derivatives of the \eqn{{}^{206}}Pb/\eqn{{}^{238}}U,
+#'     \eqn{{}^{207}}Pb/\eqn{{}^{235}}U and
+#'     \eqn{{}^{207}}Pb/\eqn{{}^{206}}Pb ratios with respect to the
+#'     intermediate decay constants and
+#'     \eqn{{}^{238}}U/\eqn{{}^{235}}U-ratio.
 #'
 #' @seealso \code{\link{diseq}}
 #' @author Noah McLean (algorithm) and Pieter Vermeesch (code)
@@ -322,38 +343,60 @@ measured.disequilibrium <- function(d=diseq()){
 #' mclean(tt=2,d=d)
 #' @export
 mclean <- function(tt=0,d=diseq(),exterr=FALSE){
-    out <- list()
-    l38 <- lambda('U238')[1]
-    l34 <- lambda('U234')[1]*1000
-    l30 <- lambda('Th230')[1]*1000
-    l26 <- lambda('Ra226')[1]*1000
-    l35 <- lambda('U235')[1]
+    out <- list(truncated=FALSE)
+    l38 <- d$L['U238']
+    l34 <- d$L['U234']
+    l30 <- d$L['Th230']
+    l26 <- d$L['Ra226']
+    l35 <- d$L['U235']
+    l31 <- d$L['Pa231']
     l32 <- lambda('Th232')[1]
-    l31 <- lambda('Pa231')[1]*1000
     U <- iratio('U238U235')[1]
     nc <- length(d$ThU$x)
-    out$dPb206U238dl38 <- rep(0,nc)
-    out$dPb206U238dl34 <- rep(0,nc)
-    out$dPb206U238dl30 <- rep(0,nc)
-    out$dPb206U238dl26 <- rep(0,nc)
-    out$dPb207U235dl35 <- rep(0,nc)
-    out$dPb207U235dl31 <- rep(0,nc)
+    rep(0,nc) -> out$dPb206U238dl38 -> out$dPb206U238dl34 ->
+        out$dPb206U238dl30 -> out$dPb206U238dl26 ->
+        out$dPb207U235dl35 -> out$dPb207U235dl31
     out$Pb208Th232 <- rep(exp(l32*tt)-1,nc)
     out$dPb208Th232dt <- rep(exp(l32*tt)*l32,nc)
-    out$d2Pb208Th232dt2 <- rep(exp(l32*tt)*l32^2,nc)
-    if (check.equilibrium(d=d)){
+    if (d$equilibrium){
         out$Pb206U238 <- rep(exp(l38*tt)-1,nc)
         out$Pb207U235 <- rep(exp(l35*tt)-1,nc)
         out$dPb206U238dt <- rep(exp(l38*tt)*l38,nc)
         out$dPb207U235dt <- rep(exp(l35*tt)*l35,nc)
-        out$d2Pb206U238dt2 <- rep(exp(l38*tt)*l38^2,nc)
-        out$d2Pb207U235dt2 <- rep(exp(l35*tt)*l35^2,nc)
         if (exterr){
             out$dPb206U238dl38 <- rep(tt*exp(l38*tt),nc)
             out$dPb207U235dl35 <- rep(tt*exp(l35*tt),nc)
         }
     } else {
-        d$n0 <- (d$n0 %*% matrix(1,nrow=1,ncol=nc)) # duplicate columns
+        nt <- meas.diseq.nt(d=d,nc=nc)[c('U238','U234','Th230','U235'),,drop=FALSE]
+        if (d$U48$option==2){
+            t234cutoff <- meas.diseq.maxt(d=d,nuclide='U234')
+            n0 <- reverse(tt=min(tt,t234cutoff),mexp=mexp.8405(),nt=nt)
+            U48i <- (n0['U234',]*l34)/(n0['U238',]*l38)
+            if (any(U48i<0)){
+                d$U48 <- list(x=0,sx=0,option=1)
+                out$truncated <- TRUE
+            }
+            if (any(U48i>d$U48$M)){
+                d$U48 <- list(x=d$U48$M,sx=0,option=1)
+                out$truncated <- TRUE
+            }
+        }
+        if (d$ThU$option==2){
+            t230cutoff <- meas.diseq.maxt(d=d,nuclide='Th230')
+            n0 <- reverse(tt=min(tt,t230cutoff),mexp=mexp.8405(),nt=nt)
+            ThUi <- (n0['Th230',]*l30)/(n0['U238',]*l38)
+            if (any(ThUi<0)){
+                d$ThU <- list(x=0,sx=0,option=1)
+                out$truncated <- TRUE
+            }
+            if (any(ThUi>d$ThU$M)){
+                d$ThU <- list(x=d$ThU$M,sx=0,option=1)
+                out$truncated <- TRUE
+            }
+        }
+        n0 <- c(1/l38,1/l34,1/l30,1/l26,0,1/l35,1/l31,0)
+        d$n0 <- (n0 %*% matrix(1,nrow=1,ncol=nc)) # duplicate columns
         rownames(d$n0) <- names(d$L)
         if (d$U48$option<2){      # initial 234U
             if (d$U48$option==1) d$n0['U234',] <- d$U48$x/l34
@@ -362,18 +405,17 @@ mclean <- function(tt=0,d=diseq(),exterr=FALSE){
             } else if (d$ThU$option==2){ # measured 230Th
                 nt <- forward(tt=tt,d=d)[c('U238','U234','Th230','U235'),,drop=FALSE]
                 nt['Th230',] <- d$ThU$x*nt['U238',]*l38/l30 # overwrite
-                d$n0['Th230',] <-
-                    reverse(tt=tt,mexp=mexp.8405(),nt=nt)['Th230',]
+                d$n0['Th230',] <- reverse(tt=tt,mexp=mexp.8405(),nt=nt)['Th230',]
             }
-        } else {                 # measured 234U
+        } else {                    # measured 234U
             if (d$ThU$option<2){ # initial 230Th
                 nt <- forward(tt=tt,d=d)[c('U238','U234','U235'),,drop=FALSE]
                 nt['U234',] <- d$U48$x*nt['U238',]*l38/l34 # overwrite
                 d$n0['U234',] <- reverse(tt=tt,mexp=mexp.845(),nt=nt)['U234',]
                 if (d$ThU$option==1) d$n0['Th230',] <- d$ThU$x/l30
-            } else {             # measured 230Th
+            } else {                # measured 230Th
                 nt <- forward(tt=tt,d=d)[c('U238','U234','Th230','U235'),,drop=FALSE]
-                nt['U234',] <- d$U48$x*nt['U238',]*l38/l34 # overwrite
+                nt['U234',] <- d$U48$x*nt['U238',]*l38/l34  # overwrite
                 nt['Th230',] <- d$ThU$x*nt['U238',]*l38/l30 # overwrite
                 d$n0[c('U234','Th230'),] <-
                     reverse(tt=tt,mexp=mexp.8405(),nt=nt)[c('U234','Th230'),]
@@ -381,25 +423,24 @@ mclean <- function(tt=0,d=diseq(),exterr=FALSE){
         }
         if (d$RaU$option>0) d$n0['Ra226',] <- d$RaU$x/l26
         if (d$PaU$option>0) d$n0['Pa231',] <- d$PaU$x/l31
-        out$U48i <- (d$n0['U234',]*l34)/(d$n0['U238',]*l38)
-        out$ThUi <- (d$n0['Th230',]*l30)/(d$n0['U238',]*l38)
         d$nt <- forward(tt=tt,d=d)
         dntdt <- forward(tt,d=d,derivative=1)
-        d2ntdt2 <- forward(tt,d=d,derivative=2)
         out$Pb206U238 <- d$nt['Pb206',]/d$nt['U238',]
         out$Pb207U235 <- d$nt['Pb207',]/d$nt['U235',]
         out$dPb206U238dt <- dntdt['Pb206',]/d$nt['U238',] -
             out$Pb206U238*dntdt['U238',]/d$nt['U238',]
-        out$d2Pb206U238dt2 <- d2ntdt2['Pb206',]/d$nt['U238',] -
-            2*dntdt['Pb206',]*dntdt['U238',]/d$nt['U238',]^2 -
-            out$Pb206U238*d2ntdt2['U238',]/d$nt['U238',] +
-            2*out$Pb206U238*(dntdt['U238',]/d$nt['U238',])^2
         out$dPb207U235dt <- dntdt['Pb207',]/d$nt['U235',] -
             out$Pb207U235*dntdt['U235',]/d$nt['U235',]
-        out$d2Pb207U235dt2 <- d2ntdt2['Pb207',]/d$nt['U235',] -
-            2*dntdt['Pb207',]*dntdt['U235',]/d$nt['U235',]^2 -
-            out$Pb207U235*d2ntdt2['U235',]/d$nt['U235',] +
-            2*out$Pb207U235*(dntdt['U235',]/d$nt['U235',])^2
+        out$U48i <- (d$n0['U234',]*l34)/(d$n0['U238',]*l38)
+        out$ThUi <- (d$n0['Th230',]*l30)/(d$n0['U238',]*l38)
+        out$RaUi <- (d$n0['Ra226',]*l26)/(d$n0['U238',]*l38)
+        out$PaUi <- (d$n0['Pa231',]*l31)/(d$n0['U235',]*l35)
+        out$U48 <- (d$nt['U234',]*l34)/(d$nt['U238',]*l38)
+        out$ThU <- (d$nt['Th230',]*l30)/(d$nt['U238',]*l38)
+        out$RaU <- (d$nt['Ra226',]*l26)/(d$nt['U238',]*l38)
+        out$PaU <- (d$nt['Pa231',]*l31)/(d$nt['U235',]*l35)
+        out$n0 <- linkUseries(n=d$n0,U=U*exp((l38-l35)*tt))
+        out$nt <- linkUseries(n=d$nt,U=U)
         if (exterr){
             K <- get.diseq.K(tt=tt,d=d)
             out$dPb206U238dl38 <- drdl(d=d,K=K,den='U238',num='Pb206',parent='U238')
@@ -415,8 +456,7 @@ mclean <- function(tt=0,d=diseq(),exterr=FALSE){
         out$dPb207Pb206dt <- (out$dPb207U235dt*out$Pb206U238 -
                               out$dPb206U238dt*out$Pb207U235)/(U*out$Pb206U238^2)
     } else {
-        out$Pb207Pb206 <- rep(0,nc)
-        out$dPb207Pb206dt <- rep(0,nc)
+        0 -> out$Pb207Pb206 -> out$dPb207Pb206dt
     }
     if (exterr){
         out$dPb207Pb206dl38 <- -out$dPb206U238dl38*out$Pb207U235/(U*out$Pb206U238^2)
@@ -425,21 +465,27 @@ mclean <- function(tt=0,d=diseq(),exterr=FALSE){
         out$dPb207Pb206dl26 <- -out$dPb206U238dl26*out$Pb207U235/(U*out$Pb206U238^2)
         out$dPb207Pb206dl35 <- out$dPb207U235dl35*out$Pb206U238/(U*out$Pb206U238^2)
         out$dPb207Pb206dl31 <- out$dPb207U235dl31*out$Pb206U238/(U*out$Pb206U238^2)
-        out$dPb208Th232dl32 <- rep(exp(l32*tt)*tt,nc)
+        out$dPb208Th232dl32 <- exp(l32*tt)*tt
         out$dPb207Pb206dU <- -out$Pb207Pb206/U
     } else {
-        out$dPb207Pb206dl38 <- rep(0,nc)
-        out$dPb207Pb206dl34 <- rep(0,nc)
-        out$dPb207Pb206dl30 <- rep(0,nc)
-        out$dPb207Pb206dl26 <- rep(0,nc)
-        out$dPb207Pb206dl35 <- rep(0,nc)
-        out$dPb207Pb206dl31 <- rep(0,nc)
-        out$dPb208Th232dl32 <- rep(0,nc)
-        out$dPb207Pb206dU <- rep(0,nc)
+        rep(0,nc) -> out$dPb207Pb206dl38 -> out$dPb207Pb206dl34 ->
+            out$dPb207Pb206dl30 -> out$dPb207Pb206dl26 ->
+            out$dPb207Pb206dl35 -> out$dPb207Pb206dl31 ->
+            out$dPb208Th232dl32 -> out$dPb207Pb206dU
     }
     out
 }
 
+linkUseries <- function(n,U){
+    num <- n[c('U238','U234','Th230','Ra226','Pb206'),,drop=FALSE]
+    den <- n['U238',,drop=FALSE]
+    U8series <- sweep(num,2,den,'/')
+    num <- n[c('U235','Pa231','Pb207'),,drop=FALSE]
+    den <- n['U235',,drop=FALSE]
+    U5series <- sweep(num,2,den,'/')
+    rbind(U*U8series,U5series)
+}
+
 get.diseq.K <- function(tt=0,d=diseq()){
     nl <- length(d$L)
     out <- matrix(0,nl,nl)
@@ -470,11 +516,9 @@ drdl <- function(tt=0,K=matrix(0,8,8),d=diseq(),
     dntdl <- (d$Q %*% P %*% d$Qinv %*% d$nt)
     rownames(dntdl) <- names(d$L)
     # 2. derivative of the ratio
-    out <- (d$nt[den,]*dntdl[num,]-
-            d$nt[num,]*dntdl[den,])/d$nt[den,,drop=FALSE]^2
-    out
+    out <- (d$nt[den,]*dntdl[num,]-d$nt[num,]*dntdl[den,])/d$nt[den,,drop=FALSE]^2
+    as.vector(out)
 }
-
 diseq.75.misfit <- function(tt,x,d){
     pred <- subset(age_to_Pb207U235_ratio(tt,d=d),select='75')
     get.5678.misfit(obs=x,pred)
@@ -491,16 +535,49 @@ get.5678.misfit <- function(obs,pred){
     abs(log(obs+1) - log(pred+1)) # +1 for robustness to negative values
 }
 
-meas.diseq.maxt <- function(d){
-    if (d$ThU$option==2){
-        out <- stats::optimise(function(tt,d) mclean(tt=tt,d=d)$ThUi^2,
-                               c(0,1),d=d)$minimum
-    } else if (d$U48$option==2){
-        M <- ifelse(d$U48$x<1,0,500)
-        out <- stats::optimise(function(tt,d,maxU48) (mclean(tt=tt,d=d)$U48i-maxU48)^2,
-                               c(0,10),d=d,maxU48=M)$minimum
+meas.diseq.maxt <- function(d,nuclide='auto'){
+    if ((d$ThU$option==2 && nuclide!='U234') || nuclide=='Th230'){
+        out <- 20*log(2)/d$L['Th230']
+    } else if (d$U48$option==2 || nuclide=='U234'){
+        out <- 20*log(2)/d$L['U234']
     } else {
         out <- 4500
     }
     out
 }
+
+measured2initial <- function(x,fit){
+    out <- x
+    fnames <- names(fit$par)
+    dnames <- c('U48','ThU','RaU','PaU')
+    for (dname in dnames){
+        iname <- paste0(dname,'i')
+        if (iname %in% fnames){
+            out$d[[dname]]$x <- fit$par[iname]
+            out$d[[dname]]$sx <- sqrt(fit$cov[iname,iname])
+            out$d[[dname]]$option <- 1
+        }
+    }
+    out
+}
+
+meas.diseq.nt <- function(d,nc=1){
+    nt <- 0*d$L
+    nt[d$L>0] <- 1/d$L[d$L>0]
+    if (d$U48$option==2){
+        nt['U234'] <- d$U48$x*nt['U238']*d$L['U238']/d$L['U234']
+    }
+    if (d$ThU$option==2){
+        nt['Th230'] <- d$ThU$x*nt['U238']*d$L['U238']/d$L['Th230']
+    }
+    out <- (nt %*% matrix(1,nrow=1,ncol=nc)) # duplicate columns
+    rownames(out) <- names(d$L)
+    out
+}
+
+pname2aname <- function(pname){
+    substr(pname, 1, nchar(pname)-1)
+}
+aname2pname <- function(aname){
+    paste0(aname,'i')
+}
